import { Component } from "@angular/core";
import { NavController, LoadingController } from "ionic-angular";
import { DefaultListable } from "../../defaults/default-listable";
import { INavigable } from "../../interfaces/navigable-interface";
import { DataProvider } from "../../providers/data-provider";
import { SaleOrderDetailsPage } from "../../pages/sale-order-details/sale-order-details";
import { SaleOrder } from "../../models/sale.order";
import { Slots } from "../../utils/slots";

@Component({
    selector: 'page-sale-order-list',
    templateUrl: 'sale-order-list.html'
})
export class SaleOrderListPage extends DefaultListable<SaleOrder> implements INavigable {

    constructor(
        public navCtrl: NavController,
        public loadingController: LoadingController,
        public db: DataProvider
    ) {
        super(loadingController);
    }

    /**
     *
     */
    ionViewDidLoad() {
        this.initialize();
    }

    /**
     *
     */
    initialize(): void {
        let loader = this.loadingCtrl.create({
            content: "Cargando presupuestos, espere.."
        });
        loader.present();

        this.db.getAll(DataProvider.DOCS.SALE_ORDER).then(results => {
            this.elements = results.filter(item => {
                return item.doc_state != "deleted";
            });


            loader.dismiss();
        }).catch(error => {
            console.log(error);
            
            loader.dismiss();
        });
    }


    /**
     *
     */
    goToPage(data: any): void {
        this.navCtrl.push(SaleOrderDetailsPage, data ? data : this);
    }
}
